(************************************************************************
              WinControl Command Declarations (General)

              This is a part outof WinControl SDK package.
              Version History:
              v1.00 - 1999/09/03*
              v1.30 - 2000/01/26 - Major Update, first public release

              Last modifyed 2000/01/26
              (c)1999-2000 HALAN Innovations, all rights reserved
              Author: Daniel Halan (daniel@wincontrol.com)

**************************************************************************)

unit kmdstd;
interface

const
  HIGH_DWORD      = 4294967295;
  FRAME_OFFSET    = 80;
  STD_CTRL_OFFSET = 40;
  STD_LABEL_OFFSET= 16;

type
  HWND = type LongWord;
  HICON = type LongWord;

  wError = (ERR_NOPROBLEM,ERR_OUT_OF_MEMORY,ERR_NO_SUCH_OBJECT,ERR_PARAMETER,
  ERR_FREELIBRARY,ERR_UNKNOWN_ORDER,ERR_UNKNOWN);
  wOSType = (osWin16,osWinNT,osWin9X,osMac,osUnix,osAmiga);
  wEventCall = (ecInitize,ecDispose);
  wOrder = (oThis,oNext,oPrev,oFirst,oLast,oRand);

  pVisible = ^wVisible;
  wVisible = record
    Item: Longint;      // -1 = always visible.. Item is what item in Array
    ItemIndex: Longint; // what itemindex should be active to show this Control
  end;

  // Only used if there is a problem with version information in Resource.
  wVer = record
    Major: Word;
    Minor: Word;
    Build: Word;
    Extra: PChar; // additional information, ie. 'Beta'
  end;

  // !!Important!!
  // Version Information is read from the Resource file, so be sure to update
  // it before release. You can do that thru "Project->Options->Version Info"
  wCmdInfo = record
    Name    : PChar;  // Plugin name, aprears in the About Dialog
    Author  : PChar;  // Your name
    Company : PChar;  // Your Company name
    VerInfo : PChar;  // Additional Version Info, ie Beta/Release
    Desc    : PChar;  // Description;  #13 = LineFeed
    HelpFile: PChar;  // Path & FileName. Only FileName = Kmd-Dir.
    Ver     : wVer;   // If there is problems with the Resouce Version this will be used
    Reserved: array[0..4] of LongWord; // Unused
  end;

  wParamType = (ptEdit,ptIntEdit,ptCombo,ptCheckbox,
                ptRadioGroup,ptBevel,ptPageControl,ptButton);
  pParamValue = ^wParamValue;
  wParamValue = record
    vType : wParamType;
    Data  : Pointer; // Visual Data Type, ie. pEdit, pComboBox
    Value : LongWord; // Int/PChar
  end;

  pParam = ^wParam;
  wParam = record
    Name        : PChar;  // Plugin Name, !Important! Don't change the name after the
                          // first release, as it is a part of the Uniqe ID your plugin
                          // gets. If you change this, all saved Trigger with your Command will be deleted.
    hIcon       : HICON;  // Default Icon Handle
    Height      : Word;     // 0 will use Default value (Default = 200)
    Width       : Word;     // 0 will use Default value (Default = 360)
    ValueCount  : Word;     // How many Parameters you will use
    Values      : array of pParamValue; // List of your parameters (Edit/IntEdit/ComboBox etc...)
  end;

  // This is the Edit-Control Values in SetupWindow
  // Requested when Updating a Trigger.
  pMainText = ^wMainText;
  wMainText = record
    Text : PChar;     // The text that will apear in the SetupWindow
    Icon : HICON;     // the Icon that will apear beside the text in SetupWindow
    ReadOnly : Boolean; // if the user can change the text or not. if TRUE the user won't
                        // have access to use the keyboard in the EditBox.
  end;

  CB_D_PChar = procedure (Str: PChar); stdcall;
  CB_N_PChar = function (Str: PChar; Length: Longint): PChar; stdcall;

  pMainValues = ^wMainValues;
  wMainValues = record          // Data from WinControl retrived at Main-Init
    WndHandle   : HWND;         // The SetupWindow Handle. use this for various API Calls
    RunOS       : wOSType;      // The Running Operating System
    NewPCharCB  : CB_N_PChar;   // Allocate New PChar
    DisposePCharCB: CB_D_PChar; // Dispose PChar allocated with NewPChar()
    Reserved    : array[0..3] of LongWord; // Don't change/use
  end;

  wButtonType = (btNone,btFile,btDir,btOther);  // wEdit/wBtn
  wEditExtras= (eNone,eFileBtn,eDirBtn);        // wEdit

  (***************************************
      WinControl Visual Components
  ****************************************)
  wOnChangeCB = function (Value: LongWord): Longint; stdcall;
                                                    // Callback that get's
                                                    // called when a user
                                                    // edits the Control.
                                                    // the Returned value
                                                    // will be used.
                                                    // if you catch this you
                                                    // MUST return an Int-value.
                                                    // If you catch this from "wEdit"
                                                    // you will recive an PChar
                                                    // if you want to change it
                                                    // you need to allocate a new
                                                    // PChar with NewPChar(), and Free
                                                    // the old Str with FreePChar().
                                                    // then Return it as Integer
                                                    // if you don't want to change the PChar, return -1.

  pVisObj = ^wVisObj;
  wVisObj = record
    Text        : PChar;    // The Caption
    PropWidth   : Byte;     // 0-100 %
    PropLeft    : Byte;     // 0-100 %
    Height      : Word;  // pixels
    TopPos      : Byte;     // 0 = topmost, 1 = 2nd from top
    HelpContext : Longint;
    VisCount    : Word;     // Count of VisObjects, 0 if always visible
    Visible     : array of pVisible; // When the Component is Visible
  end;

  pRadioGroup = ^wRadioGroup;
  wRadioGroup = record
    Text        : PChar;    // The Caption
    PropWidth   : Byte;     // 0-100 %
    PropLeft    : Byte;     // 0-100 %
    Height      : Word;  // pixels
    TopPos      : Byte;     // 1 = topmost, 2 = 2nd from top
    HelpContext : Longint;  // HelpPage to show when user press F1
    VisCount    : Word;     // Count of VisObjects, 0 if always visible
    Visible     : ^pVisible; // When the Component is Visible
    // Item Specific
    ItemCount : Word;       // How Many Items
    ItemIndex : Longint;    // Selected Index (default)
    DoOnChange: wOnChangeCB;// Callback function when user changes value
    Items     : array of PChar;
  end;

  pComboBox = ^wComboBox;
  wComboBox = record
    Text        : PChar;    // The Caption
    PropWidth   : Byte;     // 0-100 %
    PropLeft    : Byte;     // 0-100 %
    Height      : Word;     // pixels
    TopPos      : Byte;     // 1 = topmost, 2 = 2nd from top
    HelpContext : Longint;  // HelpPage to show when user press F1
    VisCount    : Word;     // Count of VisObjects, 0 if always visible
    Visible     : ^pVisible; // When the Component is Visible
    // Item Specific
    ItemCount : Word;       // How many Items
    ItemIndex : Longint;    // Selected Index (default)
    DoOnChange: wOnChangeCB;// Callback function when user changes value
    Items     : array of PChar; // List of Items
  end;

  pCheckbox = ^wCheckbox;
  wCheckbox = record
    Text        : PChar;    // The Caption
    PropWidth   : Byte;     // 0-100 %
    PropLeft    : Byte;     // 0-100 %
    Height      : Word;     // pixels
    TopPos      : Byte;     // 1 = topmost, 2 = 2nd from top
    HelpContext : Longint;  // HelpPage to show when user press F1
    VisCount    : Word;     // Count of VisObjects, 0 if always visible
    Visible     : ^pVisible; // When the Component is Visible
    // Item Specific
    Checked : Boolean;      // if Checked as default
    DoOnChange: wOnChangeCB;// Callback function when user changes value
  end;

  pEdit = ^wEdit;
  wEdit = record
    Text        : PChar;    // The Caption
    PropWidth   : Byte;     // 0-100 %
    PropLeft    : Byte;     // 0-100 %
    Height      : Word;     // pixels
    TopPos      : Byte;     // 1 = topmost, 2 = 2nd from top
    HelpContext : Longint;  // HelpPage to show when user press F1
    VisCount    : Word;     // Count of VisObjects, 0 if always visible
    Visible     : ^pVisible; // When the Component is Visible
    // Item Specific
    Value   : PChar;        // Return Value
    Extras  : wEditExtras;  //
    ExtraExt: PChar;        // ie. FileBtn = '*.exe;*.bat'
    Max     : Word;         // Max Char Length
    DoOnChange: wOnChangeCB;// Callback function when user changes value
  end;

  pIntEdit = ^wIntEdit;
  wIntEdit = record
    Text        : PChar;    // The Caption
    PropWidth   : Byte;     // 0-100 %
    PropLeft    : Byte;     // 0-100 %
    Height      : Word;     // pixels
    TopPos      : Byte;     // 1 = topmost, 2 = 2nd from top
    HelpContext : Longint;  // HelpPage to show when user press F1
    VisCount    : Word;     // Count of VisObjects, 0 if always visible
    Visible     : ^pVisible; // When the Component is Visible
    // Item Specific
    Value   : Longint; // The Value
    Min     : Longint; // Min Value
    Max     : Longint; // Max Value
    DoOnChange: wOnChangeCB;// Callback function when user changes value
  end;

  // Yet Unsupported
  pFloatEdit = ^wFloatEdit;
  wFloatEdit = record
    Text        : PChar;    // The Caption
    PropWidth   : Byte;     // 0-100 %
    PropLeft    : Byte;     // 0-100 %
    Height      : Word;     // pixels
    TopPos      : Byte;     // 1 = topmost, 2 = 2nd from top
    HelpContext : Longint;  // HelpPage to show when user press F1
    VisCount    : Word;     // Count of VisObjects, 0 if always visible
    Visible     : ^pVisible; // When the Component is Visible
    // Item Specific
    Value       : Double; // The Value
    Min         : Double; // Min Value
    Max         : Double; // Max Value
    DoOnChange: wOnChangeCB;// Callback function when user changes value
  end;

  // Yet Unsupported
  pPageControl = ^wPageControl;
  wPageControl = record
    Text        : PChar;    // The Caption
    PropWidth   : Byte;     // 0-100 %
    PropLeft    : Byte;     // 0-100 %
    Height      : Word;     // pixels
    TopPos      : Byte;     // 1 = topmost, 2 = 2nd from top
    HelpContext : Longint;  // HelpPage to show when user press F1
    VisCount    : Word;     // Count of VisObjects, 0 if always visible
    Visible     : ^pVisible; // When the Component is Visible
    // Item Specific
    ItemCount : Word;       // How many Items
    ItemIndex : Longint;    // Selected Index (default)
    DoOnChange: wOnChangeCB;// Callback function when user changes value
    Items     : array of PChar; // List of Items
  end;

function CreateEdit(pcText, pcValue: PChar; eExtra: wEditExtras; eExtraExt: PChar;
iMax, hContx: Longint; iVisCount: Word; pvArr: array of pVisible; iPropWidth,iPropLeft,iTopPos: byte;
OnVCng: wOnChangeCB = nil): pParamValue;
function CreateIntEdit(pcText: PChar; iValue,iMax,iMin,hContx: Longint;
iVisCount: Word; pvArr: array of pVisible; iPropWidth,iPropLeft,iTopPos: byte;
OnVCng: wOnChangeCB = nil): pParamValue;
function CreateComboBox(pcText: PChar; iCount: Word; pcItems: array of PChar;
iItemIndex,hContx: Longint; iVisCount: Word; pvArr: array of pVisible;
iPropWidth,iPropLeft,iTopPos: byte; OnVCng: wOnChangeCB=nil): pParamValue;
function CreateRadioGroup(pcText: PChar; iCount: Word; pcItems: array of PChar;
iItemIndex,hContx: Longint; iVisCount: Word; pvArr: array of pVisible;
iPropWidth,iPropLeft,iTopPos: Byte; iHeight: Word; OnVCng: wOnChangeCB=nil): pParamValue;
function CreateCheckBox(pcText: PChar; iValue: Boolean; hContx: Longint;
iVisCount: Word; pvArr: array of pVisible; iPropWidth,iPropLeft,iTopPos: byte; OnVCng: wOnChangeCB = nil): pParamValue;
function NewParams(pcName: PChar; iCount: Word; iHeight: Word = 0; iWidth: Word = 0; iIcon: LongWord = 0): pParam;
function FreeParams(Params: pParam): wError;
function NewArr(Count: LongWord): Pointer;
procedure DispArr(P: Pointer);
function NewPChar(Str: PChar; Length: Longint): PChar;
procedure FreePChar(Str: PChar);
procedure AssignWinCon(Main: pMainValues);
function GetPrmValue(Prm: pParamValue): Longint;
procedure SetPrmValue(Prm: pParamValue; iValue: Longint = 0; Str: PChar = nil);
function CmpPrmValue(Prm: pParamValue; iValue: Longint): Boolean;
procedure IncPrmValue(Prm: pParamValue);
procedure DecPrmValue(Prm: pParamValue);

var
  Wincon : wMainValues;

implementation // Header End
  uses
    SysUtils;
function NewPChar(Str: PChar; Length: Longint): PChar; begin Result := wincon.NewPCharCB(Str,Length) end;
procedure FreePChar(Str: PChar); begin wincon.DisposePCharCB(Str); end;
procedure DispArr(P: Pointer); begin ReAllocMem(P,0); end;
function NewArr(Count: LongWord): Pointer; var Ptr: Pointer;
begin Ptr := nil; ReallocMem( Ptr,(Count*SizeOf(Pointer)) ); Result := Ptr; end;
procedure AssignWinCon(Main: pMainValues);
begin wincon.WndHandle := Main.WndHandle; wincon.RunOS := Main.RunOS;
wincon.NewPCharCB := Main.NewPCharCB; wincon.DisposePCharCB := Main.DisposePCharCB; end;

procedure SetPrmValue(Prm: pParamValue; iValue: Longint = 0; Str: PChar = nil);
begin if Str=nil then Prm.Value := LongWord(iValue) else Prm.Value := LongWord(Str); end;

function GetPrmValue(Prm: pParamValue): Longint;
begin if Prm.vType <> ptEdit then Result := (LongWord(Prm.Value)-HIGH_DWORD)-1 else Result := 0; end;

function CmpPrmValue(Prm: pParamValue; iValue: Longint): Boolean;
begin Result := (GetPrmValue(Prm) = iValue) end;

procedure IncPrmValue(Prm: pParamValue);
begin if Prm.vType <> ptEdit then Inc(Prm.Value); end;

procedure DecPrmValue(Prm: pParamValue);
begin if Prm.vType <> ptEdit then Dec(Prm.Value); end;

function CreateParamValue(pType: wParamType; pData: Pointer; pVal: Longint = 0): pParamValue;
begin
  New(Result);
  Result.vType := pType;
  Result.Data := pData;
  Result.Value := pVal;
end;

function CreateVis(iVisCount: Word; pvArr: array of pVisible): Pointer;
  var
    x : Longint;
    VisArr: ^pVisible;
begin
  Result := NewArr(iVisCount);
  VisArr := Result;
  for x := 0 to iVisCount-1 do
    begin
      VisArr^ := pvArr[x];
      Inc(VisArr);
    end;
end;

procedure DispVis(iCount: Word; P: Pointer);
  var
    x: Longint;
    vArr: ^pVisible;
begin
  vArr := P;
  for x := 0 to iCount-1 do
    begin
      Dispose(vArr^);
      Inc(vArr);
    end;
  if (P<>nil) then DispArr(P);
end;

function CreateEdit(pcText, pcValue: PChar; eExtra: wEditExtras; eExtraExt: PChar;
iMax, hContx: Longint; iVisCount: Word; pvArr: array of pVisible; iPropWidth,iPropLeft,iTopPos: byte;
OnVCng: wOnChangeCB = nil): pParamValue;
  var
    nEdit: pEdit;
    x: Longint;
    arr: array of Pointer;
begin
  New(nEdit);
  nEdit.Text := pcText;
  nEdit.Value := pcValue;
  nEdit.Extras := eExtra;
  nEdit.ExtraExt := eExtraExt;
  nEdit.Max := iMax;
  nEdit.HelpContext := hContx;
  nEdit.VisCount := iVisCount;
  nEdit.Visible := CreateVis(iVisCount,pvArr);
  nEdit.PropWidth := iPropWidth;
  nEdit.PropLeft := iPropLeft;
  nEdit.TopPos := iTopPos;
  nEdit.DoOnChange := OnVCng;
  Result := CreateParamValue(ptEdit,nEdit);
end;

function CreateIntEdit(pcText: PChar; iValue,iMax,iMin,hContx: Longint;
iVisCount: Word; pvArr: array of pVisible; iPropWidth,iPropLeft,iTopPos: byte;
OnVCng: wOnChangeCB = nil): pParamValue;
  var
    nEdit: pIntEdit;
    x: Longint;
begin
  New(nEdit);
  nEdit.Text := pcText;
  nEdit.Value := iValue;
  nEdit.Max := iMax;
  nEdit.Min := iMin;
  nEdit.HelpContext := hContx;
  nEdit.VisCount := iVisCount;
  nEdit.Visible := CreateVis(iVisCount,pvArr);
  nEdit.PropWidth := iPropWidth;
  nEdit.PropLeft := iPropLeft;
  nEdit.TopPos := iTopPos;
  nEdit.DoOnChange := OnVCng;
  Result := CreateParamValue(ptIntEdit,nEdit);
end;

function CreateCheckBox(pcText: PChar; iValue: Boolean; hContx: Longint;
iVisCount: Word; pvArr: array of pVisible; iPropWidth,iPropLeft,iTopPos: byte; OnVCng: wOnChangeCB = nil): pParamValue;
  var
    x: Longint;
    nCheck: pCheckBox;
begin
  New(nCheck);
  nCheck.Text := pcText;
  nCheck.Checked := iValue;
  nCheck.HelpContext := hContx;
  nCheck.VisCount := iVisCount;
  nCheck.Visible := CreateVis(iVisCount,pvArr);
  nCheck.PropWidth := iPropWidth;
  nCheck.PropLeft := iPropLeft;
  nCheck.TopPos := iTopPos;
  nCheck.DoOnChange := OnVCng;
  Result := CreateParamValue(ptCheckBox,nCheck);
end;

function CreateComboBox(pcText: PChar; iCount: Word; pcItems: array of PChar;
iItemIndex,hContx: Longint; iVisCount: Word; pvArr: array of pVisible;
iPropWidth,iPropLeft,iTopPos: byte; OnVCng: wOnChangeCB=nil): pParamValue;
  var
    x : Longint;
    nCombo: pComboBox;
    VisArr: ^pVisible;
begin
  New(nCombo);
  nCombo.Text := pcText;
  nCombo.ItemCount := iCount;
  nCombo.Items := NewArr(nCombo.ItemCount);
  for x := 0 to iCount-1 do
    nCombo.Items[x] := pcItems[x];
  nCombo.ItemIndex := iItemIndex;
  nCombo.HelpContext := hContx;
  nCombo.VisCount := iVisCount;
  nCombo.Visible := CreateVis(iVisCount,pvArr);
  nCombo.PropWidth := iPropWidth;
  nCombo.PropLeft := iPropLeft;
  nCombo.TopPos := iTopPos;
  nCombo.DoOnChange := OnVCng;
  Result := CreateParamValue(ptCombo,nCombo);
end;

function CreateRadioGroup(pcText: PChar; iCount: Word; pcItems: array of PChar;
iItemIndex,hContx: Longint; iVisCount: Word; pvArr: array of pVisible;
iPropWidth,iPropLeft,iTopPos: Byte; iHeight: Word; OnVCng: wOnChangeCB=nil): pParamValue;
  var
    x : Longint;
    nRadio: pRadioGroup;
begin
  New(nRadio);
  nRadio.Text := pcText;
  nRadio.ItemCount := iCount;
  nRadio.Items := NewArr(nRadio.ItemCount);
  for x := 0 to iCount-1 do
    nRadio.Items[x] := pcItems[x];
  nRadio.ItemIndex := iItemIndex;
  nRadio.HelpContext := hContx;
  nRadio.VisCount := iVisCount;
  nRadio.Visible := CreateVis(iVisCount,pvArr);
  nRadio.PropWidth := iPropWidth; // %
  nRadio.PropLeft := iPropLeft;
  nRadio.TopPos := iTopPos;
  nRadio.Height := iHeight;
  nRadio.DoOnChange := OnVCng;
  Result := CreateParamValue(ptRadioGroup,nRadio);
end;

function NewParams(pcName: PChar; iCount: Word; iHeight: Word = 0; iWidth: Word = 0; iIcon: LongWord = 0): pParam;
begin
  New(Result);
  Result.Name := pcName;
  Result.hIcon := iIcon;
  Result.Width := iWidth;
  Result.Height := iHeight;
  Result.ValueCount := iCount;
  Result.Values := NewArr(iCount);
end;


function FreeParams(Params: pParam): wError;
  var
    x: Longint;
    Prm: pParamValue;
begin
  try
    for x := 0 to Params.ValueCount-1 do
      begin
        Prm := pParamValue(Params.Values[x]);
        DispVis(pVisObj(Prm.Data).VisCount,pVisObj(Prm.Data).Visible);
        case Prm.vType of
          ptCombo:
            begin
              DispArr(pComboBox(Prm.Data).Items);
              pComboBox(Prm.Data).Text := '';
              Dispose(pComboBox(Prm.Data));
            end;
          ptRadioGroup:
            begin
              DispArr(pRadioGroup(Prm.Data).Items);
              pRadioGroup(Prm.Data).Text := '';
              Dispose(pRadioGroup(Prm.Data));
            end;
          ptEdit:
            begin
              Wincon.DisposePCharCB(PChar(Prm.Value));
              Dispose(pEdit(Prm.Data));
            end;
          ptIntEdit: Dispose(pIntEdit(Prm.Data));
          ptCheckbox: Dispose(pCheckBox(Prm.Data));
          ptPageControl: Dispose(pPageControl(Prm.Data));
          //ptBevel: Dispose(pBevel(Params.Values[x].Data));
          end;
          Dispose(Prm);
          Params.Values[x] := nil;
      end;
    DispArr(Params.Values);
    Dispose(Params);
    Result := ERR_NOPROBLEM;
  except
    Result := ERR_FREELIBRARY;
  end;
end;

end.
