(************************************************************************
              WinControl Example Command "1"

              This is a part outof WinControl SDK package.
              Version History:
              v1.00 - 1999/09/03*
              v1.30 - 2000/01/26 - Major Update, first public release

              Last modifyed 2000/01/26
              (c)1999-2000 HALAN Innovations, all rights reserved
              Author: Daniel Halan (daniel@wincontrol.com)

**************************************************************************)

library TestCmd1;
uses
  kmdstd,
  windows,
  ShellApi,
  SysUtils,
  Classes;
{$E kmd}
{$R *.RES}

const
  // Change
  KMD_NAME = 'Delphi Example Command';

type

  wExCmd = (ecTest1,ecTest2,ecTest3);
  pCmdData = ^wCmdData;
  wCmdData = record
    Command: wExCmd;
    ExIcon: HICON;
    //Value: Longint; // Jump/Volume Step
  end;

  var
// Command specific
    Data    : pCmdData;
// Communication Var's
    Params  : pParam;
    MainOpt : pMainText;

procedure EntryPointProc(Reason: Integer);
begin
  case Reason of
    DLL_PROCESS_ATTACH:
      begin
        New(MainOpt);
        MainOpt.Text := nil;
        MainOpt.Icon := 0;
        MainOpt.ReadOnly := True;
        New(Data);
        Data.Command := ecTest1;
      end;

    DLL_PROCESS_DETACH:
      begin
        Dispose(MainOpt);
        Dispose(Data);
      end;

    DLL_THREAD_ATTACH:
      begin
      end;

    DLL_THREAD_DETACH:
      begin
      end;
  end;
end;

function GetInfo: wCmdInfo; export; stdcall;
begin
  Result.Name := KMD_NAME;
  Result.Author := 'Daniel Halan';
  Result.Company := '(c)1999-2000 HALAN Innovations';
  Result.VerInfo := 'Release';
  Result.Desc := 'Example Command for the SDK';
  Result.HelpFile := 'ExmpCmd.hlp'; // This is the HelpFile Located in the Same path as the .KMD File
end;

// example on an OnEvent Callback function
// This will be called every time the ItemIndex value
// of RadioGrp changes.
function OnRadioProc(Index: Longint): Longint; stdcall;
begin
  // do me...
  Result := Index;
end;

// Called at init and Freeing of Params.
// rMain are Callback functions for memory handling, & WinControl-WindowHandle
function Main(Event: wEventCall; rMain: pMainValues): wError; export; stdcall;
  var
    pc: PChar;
    Vis1,Vis2: pVisible;
begin
  case Event of
      ecInitize:
        begin
          AssignWinCon(rMain);
          // Add an Icon
          pc := StrAlloc(255);
          GetModuleFileName(HInstance,pc,255);
          Data.ExIcon := ExtractIcon(wincon.WndHandle,pc,0);
          StrDispose(pc);

          // Create Parameters
          Params := NewParams(KMD_NAME,
          4,            // Parameter Count
          180,          // Properties Window Height (0=standard)
          0,            // PWin Width (0=standard)
          Data.ExIcon); // Default Icon Shown in SetupWindow

          // Radio Group
          Params.Values[0] := CreateRadioGroup('Actions:',3,[
          'Test 1',
          'Test 2',
          'Test 3'],
          0,    // ItemIndex
          2,    // Help Context
          0,    // Visible Item (Param Item, RadioGroup/ComboBox)
          [],   // Visible ItemIndex (what Index should be active for thus control to be visible)
          50,   // Prop. Width, 0-100%
          0,    // Prop. Left, 0-100%
          0,    // TopPos, 0 = TopMost...
          (Params.Height-FRAME_OFFSET), // Height in Pixels
          @OnRadioProc); // OnValueChange Callback (this can be skipped if you don't use any)

        // ComboBox
          New(Vis1); Vis1.Item := 0; Vis1.ItemIndex := 0;
          Params.Values[1] := CreateComboBox('Options:',2,[
          'Something 1',  // Items
          'Something 2'],
          0,    // ItemIndex
          4,    // Help Context
          1,    // Visible Item (Param Item, RadioGroup/ComboBox)
          [Vis1],// Visible ItemIndex (what Index should be active for thus control to be visible)
          50,   // Prop. Width, 0-100%
          50,   // Prop. Left, 0-100%
          0);   // TopPos, 0 = TopMost...

          // Integer EditBox
          New(Vis1); Vis1.Item := 0; Vis1.ItemIndex := 1;
          New(Vis2); Vis2.Item := 0; Vis2.ItemIndex := 2;
          Params.Values[2] := CreateIntEdit('Test1 Integer:',129,High(Word),0,4,2,[Vis1,Vis2],50,50,0);
          // EditBox
          New(Vis1); Vis1.Item := 0; Vis1.ItemIndex := 1;
          Params.Values[3] := CreateEdit('Test2 String:','',eNone,nil,255,0,1,[Vis1],50,50,1);
          Result := ERR_NOPROBLEM;
        end;
      ecDispose:
        begin
          Result := FreeParams(Params);
        end;
          else Result := ERR_UNKNOWN_ORDER;
    end;

end;

function GetParams:pParam; export; stdcall;
begin
  Result := Params;
end;

function GetMainViewText(Order: Word; Prm: pParam): pMainText; export; stdcall;
  var
    TestPChr: PChar;
begin
  // Cast Order to wOrder
  // oThis = Current Param Value
  // oNext = Next Param Value ie: Inc(pParamValue(Prm.Values[0]).Value);
  // oPrev = Prev Param Value
  // ...
  // Order is used when the SetupWindow-Edit Box is ReadOnly, and the
  // User Presses Arrow keys Up/Down to browse thru the Options.

	{
  GetPrmValue(Prm.Values[1]) = ItemIndex of ComboBox
	GetPrmValue(Prm.Values[2]) = Value of IntEdit
	PChar(pParamValue(Prm.Values[3]).Value) = PChar of Edit. use NewPChar() / FreePChar() for manipulation
	and so on...
	}


  // All PChar's that are used by WinControl has to be allocated with NewPChar
	TestPChr := NewPChar('My PChar Test',0 {Length is used when if Str is NULL} );
	FreePChar(TestPChr);

  // Values
  //===========
  // For using the Longint value of a Param, use the XXXPrmValue() functions
  // SetPrmValue(Prm.Values[0],1);
  // GetPrmValue(Prm.Values[0]) = Value
  // CmpPrmValue(Prm.Values[0],1);
  // IncPrmValue(Prm.Values[0]);
  // DecPrmValue(Prm.Values[0]);
  // if the param is wEdit (Value is String) then Assign the Value directly
  // by Casting the PChar to LongWord. Example:
  // Prm.Values[0].Value := LongWord(NewPChar('Hello'));
  // Always Remeber, If you want to overwrite an existing Str, Free it First! (FreePChar(x))

  // Static EditBox
  Data.Command := wExCmd(GetPrmValue(Prm.Values[0])); // The RadioGroup, ItemIndex
	(*
	pParamValue(Prm.Values[1])->Value = ItemIndex of ComboBox
	pParamValue(Prm.Values[2])->Value = Value of IntEdit
	pParamValue(Prm.Values[3])->Value = PChar of Edit. use Wincon.NewPChar() / Wincon.DisposePChar() for manipulation
	and so on...
	*)

  MainOpt.Icon := Data.ExIcon; // Windows Icon Handle
  case Data.Command of
      ecTest1:  MainOpt.Text := 'Example: Some Info';
      ecTest2:  MainOpt.Text := PChar('Example: I Do Something...' + IntToStr(GetPrmValue(Prm.Values[2])));
      ecTest3:  MainOpt.Text := 'Example: I really Want to do something';
    end;
  // or Add the Text written in the TextBox
  // MainOpt.Text := PChar(pParamValue(Prm.Values[3]).Value);

  Result := MainOpt;
end;

function SetMainViewText(Prm: pParam; Str: PChar): pMainText; export; stdcall;
begin
  // Here you can Change / Approve the user change of Text, and set The Icon

  // you can modify the New "Str" that the User edits in the SetupWindow.
  // use "Wincon.NewPChar()" and "Wincon.DisposePChar()" for String Allocation/Disposing
  // use "NewArr(Count)" and "DispArr(Ptr)" to Allocate/Dispose Ptr Arrays, ie PChar
  Result := GetMainViewText(0,Prm);
end;

// Calls when the Command is Trigged
function Execute(Prm: pParam): wError; export; stdcall;
begin
  Data.Command := wExCmd(GetPrmValue(Prm.Values[0]));
  case Data.Command of
      ecTest1:
        begin
          // Do stuff..
        end;
      ecTest2:
        begin
          // Do some stuff
        end;
      ecTest3:
        begin
          // Do fun stuff...
        end;
    end;
  Result := wError(0); // OK
end;

  exports
    Execute index 0,
    GetParams index 1,
    GetMainViewText index 2,
    SetMainViewText index 3,
    Main index 4,
    GetInfo index 5;

begin
  DllProc := @EntryPointProc;
  EntryPointProc(DLL_PROCESS_ATTACH);
end.


